// Copyright 2024 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import SwiftUI
import ArcGIS


struct ContentView: View {

    @StateObject private var model = Model()

    @State private var geoViewExtent: Envelope?

    @State private var selectedCategory: Category = .coffeeShop

    @State private var tapLocation: CGPoint?
    @State private var mapLocation: Point?

    @State private var calloutPlacement: CalloutPlacement?
    @State private var calloutText: String?

    @State private var map = {
        let map = Map(basemapStyle: .arcGISTopographic)
        map.initialViewpoint = Viewpoint(latitude: 34.02700, longitude: -118.80500, scale: 72_000)
        return map
    }()

    var body: some View {

        MapViewReader { mapViewProxy in

            MapView(map: map, graphicsOverlays: [model.graphicsOverlay])

                .callout(placement: $calloutPlacement.animation(.default.speed(2))) { _ in
                    Text(calloutText ?? "No address found.")
                        .font(.callout)
                        .padding(8)
                        .frame(maxWidth: 350)
                }

                .onVisibleAreaChanged { newVisibleArea in
                    geoViewExtent = newVisibleArea.extent
                }

                .onSingleTapGesture { screenPoint, mapPoint in
                    tapLocation = screenPoint
                    mapLocation = mapPoint
                }

                .task(id: tapLocation) {
                    guard let tapLocation, let mapLocation else { return }

                    do {
                        let identifyResult = try await mapViewProxy.identify(
                            on: model.graphicsOverlay,
                            screenPoint: tapLocation,
                            tolerance: 12
                        )

                        if let graphic = identifyResult.graphics.first {
                            let placeName = graphic.attributes["PlaceName"] as? String ?? "Unknown"
                            let placeAddress = graphic.attributes["Place_addr"] as? String ?? "no address provided"
                            calloutText = "\(placeName)\n\(placeAddress)"
                            calloutPlacement = .location(mapLocation)
                        } else {
                            calloutPlacement = nil
                        }

                    } catch {
                        print(error)
                    }

                }

                .toolbar {
                    ToolbarItemGroup(placement: .bottomBar) {

                        Picker("Choose a category", selection: $selectedCategory) {
                            ForEach(Category.allCases, id: \.self) { category in
                                Text(category.label)
                            }
                        }
                        .labelsHidden()

                        .task(id: selectedCategory) {
                            await model.findPlaces(forCategory: selectedCategory, searchPoint: geoViewExtent?.center)
                        }

                    }
                }

        }

    }

}

private extension ContentView {

    enum Category: CaseIterable, Equatable {
        case coffeeShop, gasStation, food, hotel, parksOutdoors

        var label: String {
            switch self {
            case .coffeeShop: return "Coffee shop"
            case .gasStation: return "Gas station"
            case .food: return "Food"
            case .hotel: return "Hotel"
            case .parksOutdoors: return "Parks and Outdoors"
            }
        }

        var color: UIColor {
            switch self {
            case .coffeeShop: return .brown
            case .gasStation: return .orange
            case .food: return .purple
            case .hotel: return .blue
            case .parksOutdoors: return .green
            }
        }
    }

    private class Model: ObservableObject {

        let graphicsOverlay = GraphicsOverlay()

        let locator = LocatorTask(
            url: URL(string: "https://geocode-api.arcgis.com/arcgis/rest/services/World/GeocodeServer")!
        )

        func findPlaces(forCategory category: Category, searchPoint: Point? = nil) async {

            graphicsOverlay.removeAllGraphics()
            let geocodeParameters = GeocodeParameters()
            geocodeParameters.preferredSearchLocation = searchPoint
            geocodeParameters.addResultAttributeNames(["Place_addr", "PlaceName"])

            do {
                let geocodeResults = try await locator.geocode(forSearchText: category.label, using: geocodeParameters)

                if !geocodeResults.isEmpty {
                    let placeSymbol = SimpleMarkerSymbol(
                        style: .circle,
                        color: category.color,
                        size: 10
                    )
                    placeSymbol.outline = SimpleLineSymbol(
                        style: .solid,
                        color: .white,
                        width: 2
                    )
                    let graphics = geocodeResults.map { Graphic(geometry: $0.displayLocation, attributes: $0.attributes, symbol: placeSymbol) }
                    graphicsOverlay.addGraphics(graphics)
                }

            } catch {
                print(error)
            }

        }

    }

}


